@@ -12,7 +12,10 @@ from memory_mapping import readProcessMappings
 import logging
 log=logging.getLogger('model')
 
-
+#import copy
+''' replace c_char_p '''
+if ctypes.c_char_p.__name__ == 'c_char_p':
+  ctypes.original_c_char_p = ctypes.c_char_p
 
 ''' returns if the address of the struct is in the mapping area
 '''
@@ -186,7 +189,7 @@ NotNull=NotNullComparable()
 
 class CString(ctypes.Union):
   _fields_=[
-  ("string", ctypes.c_char_p),
+  ("string", ctypes.original_c_char_p),
   ("ptr", ctypes.POINTER(ctypes.c_ubyte) )
   ]
   def toString(self):
@@ -461,7 +464,7 @@ class LoadableMembers(ctypes.Structure):
       txt,full = memoryMap.readCString(attr_obj_address, MAX_SIZE )
       if not full:
         log.warning('buffer size was too small for this CString')
-      attr.string=txt
+      attr.string = txt
       return True
     else:
       _attrname='_'+attrname
@@ -569,6 +572,8 @@ class LoadableMembers(ctypes.Structure):
           s+='%s (@0x%lx) : (0x%lx) -> {%s}\n'%(field, ctypes.addressof(attr), getaddress(attr), attr.contents) # use struct printer
       elif isCStringPointer(attr):
         s+='%s (@0x%lx) : %s (CString) \n'%(field,ctypes.addressof(attr), attr.string)  
+      elif type(attr) is long or type(attr) is int:
+        s+='%s : %s\n'%(field, hex(attr) )  
       else:
         #print '*** attr cannot be __str__ ***',field, type(attr)
         s+='%s : %s\n'%(field, repr(attr) )  
@@ -693,9 +698,15 @@ def pasteLoadableMemberMethodsOn(klass):
 ''' Load all model classes and create a similar non-ctypes Python class  
   thoses will be used to translate non pickable ctypes into POPOs.
 '''
-for klass,typ in inspect.getmembers(sys.modules[__name__], inspect.isclass):
-  if typ.__module__ == __name__:
-    setattr(sys.modules[__name__], '%s_py'%(klass), type('%s_py'%(klass),(pyObj,),{}) )
+def createPOPO(modulename):
+  for klass,typ in inspect.getmembers(sys.modules[modulename], inspect.isclass):
+    if typ.__module__ == modulename:
+      setattr(sys.modules[modulename], '%s_py'%(klass), type('%s_py'%(klass),(pyObj,),{}) )
+      ##print sys.modules[modulename], '%s_py'%(klass)
+
+createPOPO(__name__)
+
 
 ''' replace c_char_p '''
-ctypes.c_char_p = CString
+if ctypes.c_char_p.__name__ == 'c_char_p':
+  ctypes.c_char_p = CString
